/* -------------------------------------------------------------------
   File "cond_plan.pro"
   ------------------------------------------------------------------- */
   Include "data.geo" ;

/* On nomme les régions */
Group{
    Domaine = Region[{AIR,INF_BOX,DIELECTRIQUE}];
    Boite_Infini = Region[{INF_BOX}];
    Air = Region[{AIR}];
    Dielectrique = Region[{DIELECTRIQUE}];

    DeltaV = Region[{CHARGE}]; /* La plaque supérieure portera la charge */
    Masse = Region[{MASSE}];

    Dirichlet = Region[{DIRICHLET}];
   /* Neumann = Region[{NEUMANN}]; */
}

/* Attribution des valeurs aux grandeurs */
Function {
    eps[#{AIR, INF_BOX}] = epsilon0;
    eps[#{DIELECTRIQUE}] = epsilon_r * epsilon0;
    Vc = POTENTIEL;
  }

/* Définition du Jacobien */
Jacobian{
    { Name Jvol;
      Case {
        { Region Boite_Infini ; Jacobian VolAxiSphShell{Rdom, Rdom+r}; } /* Jacobien volumique axisym spécifique à la boite infini */
        { Region All ; Jacobian VolAxi ; }  /* Jacobien volumique axisym pour la reste du système */
      }
    }
  }

/* Définition de la nature de l'integrale. Ici: integrale de Gauss sur les ligne et triangle (pas de quad) */
Integration {
    { Name Integ;
      Case { 
        {Type Gauss;
      Case { 
        { GeoElement Line ; NumberOfPoints 2; }
        { GeoElement Triangle ; NumberOfPoints 6; } }
        }
      }
    }
  }

/* Imposition des contraintes */
Constraint {	
    {Name TensionsImposees ;
      Case {      
        { Region Masse ; Type Assign ; Value 0. ; }   /* Tension nulle sur la plaque basse */
        { Region DeltaV ; Type Assign ; Value Vc ; }    /* Tension de Vc=1 sur la plaque haute */
        { Region Dirichlet ; Type Assign ; Value 0. ;}  /* Condition aux limites de Dirichlet, potentiel électrique nul au bord exterieur de la boite infini */
    }
    }
  }

/* Définition de l'espace fonctionnel */
FunctionSpace {
    { Name Hgrad ; Type Form0 ; /* l'espace choisi est H(grad, OMEGA) avec condition de Dirichlet à la limite exterieure */
      BasisFunction { 
        { Name sn ; NameOfCoef vn ; Function BF_Node ; /* Fonction de base = sn, les coeff = vn, fonctions nodales */
            Support Region[{Domaine}]  ; Entity NodesOf[All] ; }
      } 
      Constraint {
        { NameOfCoef vn  ; EntityType NodesOf ; NameOfConstraint TensionsImposees ; } /* Les noeuds devront respecter les conditions aux limites */
      }
    }
  }

/* Explication de la quantité à calculer */
Formulation {
    { Name Electrostat ; Type FemEquation ; /* Resolution par element finis */
      Quantity {
        { Name v  ; Type Local  ; NameOfSpace Hgrad ; } /* La variable se nomme v, elle se trouve dans H(_0)^1 */
      }
     Equation {
        Galerkin { [ eps[]*Dof{Grad v}  , {Grad v} ] ;  /* Calcul par equation de Galerkin */
          In Domaine ; Jacobian Jvol ; Integration Integ ; }  /* Dans tout le domaine, par le jacobien et l'integrale de Gauss definis precedemment */
      }
    }
  }

/* Lancement de la resolution, des calculs */
Resolution {
    { Name analyse ;
      System {
        { Name Sys ; NameOfFormulation Electrostat ; }
      }
      Operation {
        Generate[Sys] ; 
        Solve[Sys] ;
        SaveSolution[Sys] ;
      }
    }
  }

/* A partir de la resolution, on va calculer les quantites qui nous interessent */
PostProcessing {
    { Name PostProc ; NameOfFormulation Electrostat ;
      Quantity {	
        { Name v  ; Value { Term { [ {v} ] ; In Domaine ; Jacobian Jvol ; } } }   /* Les potentiels v qui correspondent à la grandeur calcule lors de la resolution */
        { Name e  ; Value { Term { [ -{Grad v} ] ; In Domaine ; Jacobian Jvol ; } } }   /* le champ electrique */
        { Name d  ; Value { Term { [ -eps[]*{Grad v} ] ; In Domaine ; Jacobian Jvol ; } } }   /* l'induction electrique */
        { Name nrj ; Value { Integral { [ 2*Pi*eps[]*SquNorm[{Grad v}] ]  ; In Domaine ;    
          Jacobian Jvol ; Integration Integ ;} } } /* l'energie electrique, integrale volumique d'ou le 2*pi pour prendre en compte l'axisymetrie */
        { Name C ; Value { Integral { [ 4*Pi*eps[]*SquNorm[{Grad v}]/(Vc^2) ]  ; In Domaine ;   
          Jacobian Jvol ; Integration Integ ;} } } /* la capacité du condensateur */
      }
    }
  }

/* Affichage des fleches et des isovaleurs */
PostOperation PostOp UsingPost PostProc {
    Print[ v, OnElementsOf Domaine, File "v.pos"] ;
    Print[ d, OnElementsOf Domaine, File "d.pos"] ;
    Print[ e, OnElementsOf Domaine, File "e.pos"] ;
    Print[ nrj[Domaine], OnGlobal, File "nrj.dat", 
      SendToServer "1Energie (J)", Color "AliceBlue" ];   /* C:\Users\Hunte\Desktop\ENSEM\S8\B7\EC2b\condo_plan\  */
    Print[ C[Domaine], OnGlobal, File "capa.dat", 
      SendToServer "2Capa (F)", Color "AliceBlue" ];    /* C:\Users\Hunte\Desktop\ENSEM\S8\B7\EC2b\condo_plan\  */
  }

